Package: ribosomeProfilingQC
Type: Package
Title: Ribosome Profiling Quality Control
Version: 1.25.0
Authors@R: c(person(given="Jianhong", family="Ou", email="jou@morgridge.org", 
        role=c("aut", "cre"), comment=c(ORCID="0000-0002-8652-2488")),
        person(given="Mariah", family="Hoye", email="mariah.hoye@duke.edu",
        role=c("aut")))
Description: Ribo-Seq (also named ribosome profiling or footprinting) 
        measures translatome (unlike RNA-Seq, which sequences the transcriptome)
        by direct quantification of the ribosome-protected fragments (RPFs). 
        This package provides the tools for quality assessment of ribosome 
        profiling. In addition, it can preprocess Ribo-Seq data for subsequent
        differential analysis. 
License: GPL (>=3) + file LICENSE
Encoding: UTF-8
biocViews: RiboSeq, Sequencing, GeneRegulation, QualityControl,
        Visualization, Coverage
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Depends: R (>= 4.0), GenomicRanges
Imports: AnnotationDbi, BiocGenerics, Biostrings, BSgenome, EDASeq,
        GenomicAlignments, GenomicFeatures, Seqinfo, GenomeInfoDb,
        IRanges, methods, motifStack, rtracklayer, Rsamtools, RUVSeq,
        Rsubread, S4Vectors, XVector, ggplot2, ggfittext, scales,
        ggrepel, utils, cluster, stats, graphics, grid, txdbmaker,
        ggExtra
Suggests: RUnit, BiocStyle, knitr, BSgenome.Drerio.UCSC.danRer10,
        GenomeInfoDbData, edgeR, DESeq2, limma, ashr, testthat,
        rmarkdown, vsn, Biobase
git_url: https://git.bioconductor.org/packages/ribosomeProfilingQC
git_branch: devel
git_last_commit: a19b5ff
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:44:10 UTC; biocbuild
Author: Jianhong Ou [aut, cre] (ORCID: <https://orcid.org/0000-0002-8652-2488>),
  Mariah Hoye [aut]
Maintainer: Jianhong Ou <jou@morgridge.org>
