# ribosomeProfilingQC
An R package for quality assessment of ribosome profiling

## Installation

To install this package, start R and enter:

```r
library(BiocManager)
BiocManager::install("jianhong/ribosomeProfilingQC")
```

## Documentation

To view documentation of ribosomeProfilingQC, start R and enter:
```r
browseVignettes("ribosomeProfilingQC")
```
