require("ribosomeProfilingQC") || stop("unable to load Package:ribosomeProfilingQC")
require("GenomicRanges") || stop("unable to load Package::GenomicRanges")
require("GenomicFeatures") || stop("unable to load Package::GenomicFeatures")
require("GenomicAlignments") || stop("unalbe to load Package:GenomicAlignments")
require("utils") || stop("unable to load Package:utils")
require("AnnotationDbi") || stop("unable to load Package:AnnotationDbi")
require("BSgenome.Drerio.UCSC.danRer10") || stop("unable to load Package:BSgenome.Drerio.UCSC.danRer10")
require("Rsamtools") || stop("unable to load Rsamtools")
require("rtracklayer") || stop("unable to load rtracklayer")
require("testthat") || stop("unable to load testthat")
test_check("ribosomeProfilingQC")
